/***************************************************************************
                             ksettingsreports.cpp
                             --------------------
    copyright            : (C) 2010 by Bernd Gonsior
    email                : bernd.gonsior@googlemail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// ----------------------------------------------------------------------------
// QT Includes

// ----------------------------------------------------------------------------
// KDE Includes

// ----------------------------------------------------------------------------
// Project Includes

#include "ksettingsreports.h"

class KSettingsReports::Private
{
public:

  Private(KMyMoneyApp* kmymoneyapp) {
    m_kmymoneyapp = kmymoneyapp;
  }

  ~Private() {}

  /**
   * Collector for both signals
   * urlSelected and editingFinished.
   *
   * Only shows a warning
   * if the selected file
   * is not a readable plain file -
   * and only one time.
   *
   * @param[in] css  css file name
   *
   * @see KSettingsReports#slotCssUrlSelected
   * @see KSettingsReports#slotEditingFinished
   */
  void checkCssFile(QString& css) {

    if (css == m_cssFileOld) {
      // do not check again to avoid emitting a warning more than 1 time
      return;
    }

    // store the current filename for later comparison
    m_cssFileOld = css;

    // list for possible error/warn messages
    QList<QString> warnings;

    if (m_kmymoneyapp->checkOptionCssFile(css, warnings)) {
      // ok, nothing else to do
      qDebug() << "DBG: KSettingsReports::Private::checkCssFile: checkOptionCssFile succeeded for:" << css;
      return;
    }

    // there was at least 1 warning
    QString out = i18np("There is a problem with file %2",
                        "There are problems with file %2", warnings.size(), css);

    QList<QString>::const_iterator i;
    for (i = warnings.constBegin(); i != warnings.constEnd(); ++i) {
      out += '\n' + *i;
    }

    // this must go to as a message to the user and/or into an error log
    KMessageBox::sorry(0, out);
    qDebug() << "WRN: KSettingsReports::Private::checkCssFile: warnings: " << out;
  }

  /**
   * Old value of css file to avoid warnings
   * when a signal is emitted
   * but the value itself did not change.
   */
  QString m_cssFileOld;

  /**
   * Pointer to the KLineEdit of the KFileDialog which we need
   * to receive signal editingFinished.
   */
  KLineEdit* m_fileKLineEdit;

  KMyMoneyApp* m_kmymoneyapp;
};

KSettingsReports::KSettingsReports(KMyMoneyApp* kmymoneyapp, QWidget* parent) :
    KSettingsReportsDecl(parent),
    d(new Private(kmymoneyapp))
{

  // keep initial (default) css file in mind
  d->m_cssFileOld = KMyMoneyGlobalSettings::cssFileDefault();

  // set default css file in ksettingsreports dialog
  kcfg_CssFileDefault->setUrl(KMyMoneyGlobalSettings::cssFileDefault());

  d->m_fileKLineEdit = kcfg_CssFileDefault->lineEdit();

  connect(kcfg_CssFileDefault, SIGNAL(urlSelected(const KUrl&)),
          this, SLOT(slotCssUrlSelected(const KUrl&)));

  connect(d->m_fileKLineEdit, SIGNAL(editingFinished()),
          this, SLOT(slotEditingFinished()));
}

KSettingsReports::~KSettingsReports()
{
  delete d;
}

/**
 * Receiver for signal urlSelected.
 *
 * Signal urlSelected only is emitted
 * when a file is selected with the file chooser.
 *
 * @param[in] cssUrl  url of css file
 *
 * @see KSettingsReports#Private#checkCssFile
 */
void KSettingsReports::slotCssUrlSelected(const KUrl& cssUrl)
{
  QString css = cssUrl.isLocalFile() ? cssUrl.toLocalFile() : cssUrl.url();
  d->checkCssFile(css);
}

/**
 * Receiver for signal editingFinished.
 *
 * Signal editingFinished is emitted
 * on focus out only,
 * not  when a file is selected with the file chooser.
 *
 * @see KSettingsReports#Private#checkCssFile
 */
void KSettingsReports::slotEditingFinished()
{
  QString txt = d->m_fileKLineEdit->text();
  d->checkCssFile(txt);
}

#include "ksettingsreports.moc"
